;#####################################################################################
;
; (C) Cooperative Institute for Meteorological Satellite Studies, 2003
;
; NAME: eff_wn.pro
;
; NOTES: This procedure file contains functions to calculate constants used
;        to calculate brightness temperature using an effective wavenumber
;        unique to the SRF being used opposed to calculating brightness
;        temperature monochromatically.  The functions used are from Hal
;        Woolf (CIMSS/SSEC) and Mat Gunshor (CIMSS/SSEC).
;
;#####################################################################################



FUNCTION plncon, wn

;--Boltzmann
b=1.3806503D-16
;--Planck
h=6.62606876D-27
;--Light
c=2.99792458D+10

c1=2.0*h*c*c
c2=h*c/b

fk1=c1*wn^3
fk2=c2*wn

fks=[fk1,fk2]
return, fks

END

FUNCTION wplanc, vn, tem

;--Boltzmann
b=1.3806503D-16
;--Planck
h=6.62606876D-27
;--Light
c=2.99792458D+10

c1=2.0*h*c*c
c2=h*c/b
f1=c1*vn^3
f2=c2*vn
t=tem
rad=f1/(exp(f2/t)-1)

return, rad

END

FUNCTION wbrite, vn, rad

;--Boltzmann
b=1.3806503D-16
;--Planck
h=6.62606876D-27
;--Light
c=2.99792458D+10

c1=2.0*h*c*c
c2=h*c/b
f1=c1*vn^3
f2=c2*vn

r=rad
tbb=f2/alog(f1/r+1)

return, tbb

END


FUNCTION linreg, x, y, n

xbar=0
ybar=0
for i=0,n-1 do begin
	xbar=xbar+x[i]
	ybar=ybar+y[i]
endfor

xbar=xbar/n
ybar=ybar/n
xvar=0
yvar=0
cvar=0

for i=0,n-1 do begin
	xp=x[i]-xbar
	yp=y[i]-ybar
	xvar=xvar+xp*xp
	yvar=yvar+yp*yp
	cvar=cvar+xp*yp
endfor

xvar=xvar/n
xsig=sqrt(xvar)
yvar=yvar/n
ysig=sqrt(yvar)
cvar=cvar/n
slope=cvar/xvar
yint=ybar-slope*xbar
corr=cvar/(xsig*ysig)
evar=slope*cvar
serr=sqrt(abs(yvar-evar))
evar=evar/yvar

lin=[yint,slope]

return, lin
	

END


FUNCTION bndfit, w,r,np,tl,tu,wc

npts=100
;----Establish 100 point equal spacing in B between TL and TU
bl=wplanc(wc,tl)
bu=wplanc(wc,tu)
db=(bu-bl)/99.0

;----Find100 equivalent temperatures TE
te=fltarr(100)
for i=0,npts-1 do begin
	b=bl+db*i
	te[i]=wbrite(wc,b)
endfor

;---Find true radiances at TE and inverse Planck temps
ti=fltarr(100)
for j=0, npts-1 do begin
	a=0.0
	b=0.0
	t=te[j]
	for i=0,np-2 do begin
		ip1=i+1
		dw=w[ip1]-w[i]
		wx=w[i]+0.5*dw
		rx=0.5*(r[i]+r[ip1])
		a=a+dw*wplanc(wx,t)*rx
		b=b+dw*rx
	endfor
	rad=a/b
	ti[j]=wbrite(wc,rad)
endfor

;---Obtain least-squares fit for TI vs. TE
lsf=[0.0,0.0]
lsf=linreg(te,ti,npts)

return, lsf

END


FUNCTION EFF_WN, pick_srf, center_freq

openr, lun, pick_srf, /get_lun
nlines=0
while (not eof(lun)) do begin
	readf, lun, temp
	nlines = nlines + 1
endwhile
free_lun, lun
data = fltarr(2,nlines)
openr, lun, pick_srf, /get_lun
readf, lun, data
free_lun, lun
in_srf = data[1,*]
in_freq = data[0,*]

sumr=0.0
sumw=0.0
for i=0,nlines-1 do begin
	rr = in_srf[i]
	sumw=sumw+(rr*in_freq[i])
	sumr=sumr+rr
endfor
ww=0.0
ww=sumw/sumr
cwn=ww
center_wavel=center_freq/10000
if(center_wavel le 4.10) then tu=350.0
if(center_wavel gt 4.10 and center_wavel le 7.50) then tu=310.0
if(center_wavel gt 7.50 and center_wavel le 8.80) then tu=320.0
if(center_wavel gt 8.80 and center_wavel le 12.50) then tu=330.0
if(center_wavel gt 12.50) then  tu=300.0
tl=180.0

;-----------GET FK1 and FK2
fk=[0.0,0.0]
fk=plncon(cwn)
;--------GET BC1 and BC2
bcc=[0.0,0,0]
bcc=bndfit(in_freq,in_srf,nlines,tl,tu,ww)

constants=[fk[0],fk[1],bcc[0],bcc[1]]
return, constants

END

