; PROGRAM NAME: read_swath_l1_airs.pro
;
; PURPOSE: read data from an AIRS IR Level-1B file (ESDT name = AIRIBRAD). 
;
;
; CATEGORY: AIRS / Level-1B / Reader
;
;
; INPUTS:
;         pattern = an AIRS Level-1B filename.
;
;
; OUTPUTS:
;         numfp = number of footprints per scan line.
;         numline = number of lines in this granule.
;         tai[numline] = TAI (International Atomic Time), seconds since 1/1/1993
;         lat[numfp,numline] = footprint boresight latitude.
;         lon[numfp,numline] = footprint boresight longitude.
;         rad[2378,numfp,numline] = radiance (milliWatts/m^2/cm^-1/steradian).
;         solzen[numfp,numline] = solar zenith angles in degrees.
;
;
;         Optional:
;
;         scanang[numfp,numline] = scan angle (<0 at start of scan, 0 at nadir).
;
;         sun_glint_distance[numfp,numline] = distance in km between footprint 
;                                             and center location of the sun 
;                                             glint.
;
;         CalFlag[2378,numline] = Calibration status. Bit field by channel for
;                                 the current scan line. 0 means the channel 
;                                 is well calibrated. If it is not 0, you
;                                 need to decode the individual bits. See
;                                 L1B_AIRS_Science Interface Spec document.
;
;         freq[2378] = channel frequencies (wavenumber) in cm-1.
;
;         topog[numfp,numline] = mean topograpy in meters above ref ellipsoid. 
;
;         state[numfp,numline] = data set state. 0:process; 1:special;
;                                2:Erroneous; 3:missing
;
;
; REQUIRED SUBPROGRAMS: none
;
;
; LIMITATIONS: 
;       1. relies on HDFEOS routines to read contents of an input file. 
;          HDFEOS routines in earlier versions of IDL may not work properly. 
;
;       2. read data at once, no option for reading a slice of data. Therefore
;          memory requirement is very high ~ 120 MB.
;...............................................................................
; Programming Language: IDL 5.4 or later
;
; Modification History:
; $Log: read_swath_l1_airs.pro,v $
; Revision 1.2  2003/03/18 22:46:44  jyli
; add comments for public release.
;
; Revision 1.1  2003/03/18 22:41:19  jyli
; Initial revision
;
;...............................................................................
; $Id: read_swath_l1_airs.pro,v 1.2 2003/03/18 22:46:44 jyli Exp jyli $
; 
; PROGRAMMER:
;            NASA/JPL
;===============================================================================

pro read_swath_l1_airs, pattern, numfp, numline, tai, lat, lon, rad, solzen, freq,$
                        scanang=scanang, $
                        sun_glint_distance=sun_glint_distance, $
                        CalFlag=CalFlag, $
                        topog=topog, state=state

   filename = findfile ( pattern, count = cnt )
   if cnt ne 1 then begin
      print, 'findfile did not return exactly one file ', cnt, pattern
      return
   endif

   print, 'read_swath_l1_airs: ', filename[0]

   FAIL = -1
   SWATH_L1AIRS = 'L1B_AIRS_Science'
   cc_level1 = 0

   fid=EOS_SW_OPEN(filename[0], /READ)
   if fid eq FAIL then begin
      return
   endif

   SWid=EOS_SW_ATTACH(fid, SWATH_L1AIRS)
   if SWid eq FAIL then begin
      SWATH_L1AIRS = 'L2_Standard_cloud-cleared_radiance_product'
      SWid=EOS_SW_ATTACH(fid, SWATH_L1AIRS)
      cc_level1 = 1
   endif

   ret = EOS_SW_READFIELD(SWid,"Time",tai)
   sz=size(tai)
   numfp=sz[1]
   numline=sz[2]

   ret = EOS_SW_READFIELD(SWid,"Latitude",lat)
   ret = EOS_SW_READFIELD(SWid,"Longitude",lon)
   ret = EOS_SW_READFIELD(SWid,"solzen",solzen)
   ret = EOS_SW_READFIELD(SWid,"radiances",rad)
   ret = EOS_SW_READFIELD(SWid,"spectral_freq",freq)

   if arg_present(scanang) then begin
      print, "reading the scan angle"
      ret = EOS_SW_READFIELD(SWid,"scanang",scanang)
   endif

   if arg_present(sun_glint_distance) then begin
      print, "reading the sun glint distance"
      ret = EOS_SW_READFIELD(SWid,"sun_glint_distance",sun_glint_distance)
   endif

   if arg_present(CalFlag) then begin
      print, "reading the CalFlag"
      ret = EOS_SW_READFIELD(SWid,"CalFlag",CalFlag)
   endif

;   if arg_present(freq) then begin
;      print, "reading the frequencies"
;      if cc_level1 eq 0 then begin
;	 ret = EOS_SW_READFIELD(SWid,"spectral_freq", freq )
;      endif else begin
;         ret = EOS_SW_READFIELD(SWid,"freq", freq )
;      endelse
;   endif

   if arg_present(state) then begin
      print, "reading the state flag"
      ret = EOS_SW_READFIELD(SWid,"state",state)
   endif

   if arg_present(topog) then begin
      print, "reading the topography"
      ret = EOS_SW_READFIELD(SWid,"topog",topog)
   endif

   print, "done reading - detaching and closing the file"
   ret = EOS_SW_DETACH(SWid)
   ret = EOS_SW_CLOSE(fid)

end
